Package: IRanges
Title: Foundation of integer range manipulation in Bioconductor
Description: Provides efficient low-level and highly reusable S4
	classes for storing, manipulating and aggregating over annotated
	ranges of integers. Implements an algebra of range operations,
	including efficient algorithms for finding overlaps and nearest
	neighbors. Defines efficient list-like classes for storing,
	transforming and aggregating large grouped data, i.e., collections
	of atomic vectors and DataFrames.
biocViews: Infrastructure, DataRepresentation
URL: https://bioconductor.org/packages/IRanges
BugReports: https://github.com/Bioconductor/IRanges/issues
Version: 2.46.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("Patrick", "Aboyoun", role="aut"),
	person("Michael", "Lawrence", role="aut"))
Depends: R (>= 4.0.0), methods, utils, stats, BiocGenerics (>= 0.53.2),
        S4Vectors (>= 0.47.6)
Imports: stats4
LinkingTo: S4Vectors
Suggests: XVector, GenomicRanges, Rsamtools, GenomicAlignments,
        GenomicFeatures, BSgenome.Celegans.UCSC.ce2, pasillaBamSubset,
        RUnit, BiocStyle
Collate: thread-control.R range-squeezers.R Vector-class-leftovers.R
        DataFrameList-class.R DataFrameList-utils.R AtomicList-class.R
        AtomicList-utils.R Ranges-and-RangesList-classes.R
        IPosRanges-class.R IPosRanges-comparison.R
        IntegerRangesList-class.R IRanges-class.R IRanges-constructor.R
        makeIRangesFromDataFrame.R IRanges-utils.R
        Rle-class-leftovers.R IPos-class.R subsetting-utils.R
        Grouping-class.R Views-class.R RleViews-class.R
        RleViews-summarization.R extractList.R seqapply.R multisplit.R
        SimpleGrouping-class.R IRangesList-class.R IPosList-class.R
        ViewsList-class.R RleViewsList-class.R RleViewsList-utils.R
        RangedSelection-class.R MaskCollection-class.R read.Mask.R
        CompressedList-class.R CompressedList-comparison.R
        CompressedHitsList-class.R CompressedDataFrameList-class.R
        CompressedAtomicList-class.R
        CompressedAtomicList-summarization.R CompressedGrouping-class.R
        CompressedRangesList-class.R Hits-class-leftovers.R
        NCList-class.R findOverlaps-methods.R windows-methods.R
        intra-range-methods.R inter-range-methods.R reverse-methods.R
        coverage-methods.R cvg-methods.R slice-methods.R
        setops-methods.R nearest-methods.R cbind-Rle-methods.R
        tile-methods.R extractListFragments.R zzz.R
git_url: https://git.bioconductor.org/packages/IRanges
git_branch: RELEASE_3_23
git_last_commit: db589cc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:48:08 UTC; biocbuild
Author: Hervé Pagès [aut, cre],
  Patrick Aboyoun [aut],
  Michael Lawrence [aut]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
