\name{IRanges internals}

\alias{coerce,ANY,vector-method}


\title{IRanges internals}

\description{
  Objects, classes and methods defined in the \pkg{IRanges} package
  that are not intended to be used directly.
}

\keyword{internal}
\keyword{classes}
\keyword{methods}
